function [Oqeggmes,f0mes,npicferm,npicouver] = decom(s,fs,f0,choix)
%DECOM  Mesure de f0 et Oq sur le signal DEGG
% 	[Oqeggmes,f0mes,npicferm,npicouver] = decom(s,fs,f0,choix)
%	mesure la frquence fondamentale f0mes, le quotient d'ouverture Oqeggmes,
%	le nombre de pics  la fermeture npicferm (vrification de l'unicit du pic),
%	le nombre de pics  l'ouverture npicouver, sur un signal DEGG,
%	connaissant les bornes en frquence f0 =  [f0min,f0max] et la frquence
%	d'chantillonnage fs. Par dfaut, f0min = 80, f0max = 1500.
%   Si f0 ne donne qu'une valeur, elle est directement utilise pour le
%   calcul.
%	La variable choix dfinit la mthode utilise pour le
%	calcul de Oq : 'max', le pic de la fonction
%	d'intercorrlation ayant une amplitude maximale est
%	slectionn, 'premier' le premier des pics de la fonction
%	d'intercorrlation est slectionn, 'dernier' le dernier
%	des pics est slectionn.

%	Auteur : Nathalie Henrich, 04/04/06.
%	D'aprs DeggMesure.m


if nargin <3
    f0min = 80;
    f0max = 1500;
elseif length(f0)==2
    f0min = f0(1);
    f0max = f0(2);
end

if nargin < 5
    choix = 'max';
end

% initialisation des tableaux
indn=[];
indp=[];
ep=[];
en=[];
interc=[];
autoc=[];

% valeur par dfaut
npicferm = -1;
npicouver = -1;


% separation partie positive / partie negative
indn = find(s < 0);
indp = find(s > 0);
ep = s;
en = - s;
ep(indn) =0;
en(indp) =0;

% longueur des vecteurs s, ep et en
L = length(s);


%%%%%%%%%%%%%%%
% CALCUL DE F0
%%%%%%%%%%%%%%%

if length(f0) == 1
    f0mes = f0;
    % prcision de 0.5 Hz
    deltat = 0.5 / f0mes^2;

else

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % calcul de l'autocorrelation normalise (et biaise)
    % (autocorrlation non circulaire)
    %
    % autre mthode (autocorrlation circulaire)
    %
    % autoc = real(ifft(abs(fft(ep,2*L)).^2)) / L;
    % autoc = autoc(1:L);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    autoc = xcorr(ep,'coeff');
    autoc = autoc(L:2*L-1);

    % dtection du pic correspondant  la priode
    % seuil de 0.5 (critre vois / non vois)
    [maxa,indmaxa]=PicDetect(autoc,0.5);

    if isempty(maxa)
        f0mes=0;
        Oqeggmes=0;
    elseif autoc(max(1,indmaxa(1)-4))>=autoc(indmaxa(1))
        f0mes=0;
        Oqeggmes=0;
    else

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % estimation de f0
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % estimation grossire
        f0approx=fs/(indmaxa(1)-1);

        % prcision de 0.5 Hz
        deltat = 0.5 / f0approx^2;

        % interpolation cubique entre 5 points
        % centrs autour du pic dtect

        % slection temporelle
        indsel = [max(1,indmaxa(1)-2):indmaxa(1) indmaxa(1)+1 indmaxa(1)+2];
        tsel = (indsel-1)/fs;
        autocsel = autoc(indsel);
        % rchantillonnage
        ti = min(tsel):deltat:max(tsel);
        % interpolation
        autoci = interp1(tsel,autocsel,ti,'spline');
        % dtection du maximum
        [ymax1,indmax1] = max(autoci);

        % estimation de f0  0.5 Hz prs
        t1 = ti(indmax1);
        f0mes = 1/t1;


        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % vrification des bornes en frquence
        % de f0
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        if ((f0mes>=f0max)|(f0mes<=f0min))
            f0mes=0;
            Oqeggmes=0;
        end
    end
end



%%%%%%%%%%%%%%%
% CALCUL DE OQ
%%%%%%%%%%%%%%%

if f0mes == 0
    Oqeggmes = 0;

else

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % vrification de l'unicit du pic
    % correspondant  l'ouverture
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    npicouver = NombredePics(en,f0mes,fs,0.7,40);


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % calcul de l'intercorrelation
    % entre le signal positif (fermeture)
    % et le signal ngatif (ouverture)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    interc = xcorr(ep,en,'coeff'); % DIVA
    interc = interc(L:2*L-1);

    % dtection du (des) pic(s) correspondant
    % seuil  0.8
    [maxid,indmaxid]=PicDetect(interc,0.75);

    [maxi,indi] = max(maxid);

    if isempty(indi)
        Oqeggmes = 0;
        return;
    end

    if npicouver == 1
        indmaxi = indmaxid(indi);

    else
        switch choix
            case 'max'

                % slection du pic ayant une amplitude maximale
                indmaxi = indmaxid(indi);

            case 'premier'
                indmaxi = min(indmaxid);

            case 'dernier'
                indmaxi = max(indmaxid);
        end
    end


    % dtection prcise du maximum
    % slection temporelle
    indsel = [max(1,indmaxi(1)-2):indmaxi(1) indmaxi(1)+1 indmaxi(1)+2];
    tsel = (indsel-1)/fs;
    intercsel = interc(indsel);

    % rchantillonnage
    ti = min(tsel):deltat:max(tsel);
    % interpolation
    interci = interp1(tsel,intercsel,ti,'spline');
    % dtection du maximum
    [ymax2,indmax2] = max(interci);


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % valeur de Oq mesure
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    t2 = ti(indmax2);
    Oqeggmes= t2 * f0mes;


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % vrification de l'unicit du pic
    % correspondant  la fermeture
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    npicferm = NombredePics(ep,f0mes,fs,0.5,40);


    if Oqeggmes >1
        Oqeggmes = 0;
    end


end



